#include "stdio.h"
#include "stm8s.h"
#include "usrlib-uart.h"

#define fMASTER    16000000  // 16MHz

void main(void)
{
	char TXStr[100];

// clock setting
	CLK_ClockSwitchConfig(CLK_SWITCHMODE_AUTO, CLK_SOURCE_HSI, DISABLE, CLK_CURRENTCLOCKSTATE_DISABLE);
	CLK_HSIPrescalerConfig( CLK_PRESCALER_HSIDIV1 );
	CLK_SYSCLKConfig( CLK_PRESCALER_CPUDIV1 );
  
	// display indicator
	GPIO_Init(GPIOD, GPIO_PIN_0, GPIO_MODE_OUT_PP_HIGH_FAST);
  
  // frequency counter input port  TIM1_CH1 PC1
	TIM1_DeInit();
	TIM1_PrescalerConfig(0, TIM1_PSCRELOADMODE_UPDATE);
	TIM1_PWMIConfig(TIM1_CHANNEL_1, TIM1_ICPOLARITY_RISING, TIM1_ICSELECTION_DIRECTTI, TIM1_ICPSC_DIV1, 0);
	TIM1_SelectInputTrigger( TIM1_TS_TI1FP1 );
	TIM1->SMCR = (TIM1->SMCR & ~TIM1_SMCR_SMS) | 0b100; // TIM1->SMCR->SMS=100 (Reset mode)
  TIM1_ITConfig(TIM1_IT_CC1, ENABLE);
  TIM1_Cmd(ENABLE);

	// 1kHz 30% duty oscillator
	// 1kHz ---> 16MHz/1000 = 16000
	// 30% duty ---> 16000*0.3 = 4800
  // TIM2_CH1 D4 _______|~~~~~~|____________|~~~~~~|_______
  // TIM2_CH2 D3 ~~~~~~~|______|~~~~~~~~~~~~|______|~~~~~~~
	TIM2_DeInit();
	TIM2_TimeBaseInit(TIM2_PRESCALER_1, 15999);
	TIM2_OC1Init(TIM2_OCMODE_PWM1, TIM2_OUTPUTSTATE_ENABLE, 4800, TIM2_OCPOLARITY_HIGH); // output TIM2_CH1 D4
	TIM2_OC2Init(TIM2_OCMODE_PWM2, TIM2_OUTPUTSTATE_ENABLE, 4800, TIM2_OCPOLARITY_HIGH); // output TIM2_CH2 D3
	TIM2_Cmd(ENABLE);

  // display timebase
	TIM3_DeInit();
  TIM3_TimeBaseInit(TIM3_PRESCALER_1024 , 10000); // 16MHz/1024/10000=1.56Hz
  TIM3_ITConfig(TIM3_IT_UPDATE, ENABLE);
  TIM3_Cmd(ENABLE);
	
	UART2_DeInit();
  UART2_Init((u32)115200, UART2_WORDLENGTH_9D, UART2_STOPBITS_1, UART2_PARITY_ODD, UART2_SYNCMODE_CLOCK_DISABLE, UART2_MODE_TXRX_ENABLE);	
  UART2_ITConfig(UART2_IT_RXNE_OR, ENABLE);
	UART2_Cmd(ENABLE);
	
	enableInterrupts();
	
	sprintf(TXStr,"start\r\n");
	SerialPutString(TXStr);
	
  while (1)	{	}
}

// TIM1 interrupt routine
// get frequency counter values
//  __________|~~~~~~|____________|~~~~~~|__________
//            <------------------->  TIM1_ICvalue1
//            <------>   TIM1_ICvalue2
u16 TIM1_IC_value1=0, TIM1_IC_value2=0;
void TIM1_IC_isr(void) interrupt 12
{
	TIM1_IC_value1 = TIM1_GetCapture1()+1;  // get value
	TIM1_IC_value2 = TIM1_GetCapture2()+1;  // get value
	TIM1_ClearFlag(TIM1_FLAG_CC1);
	TIM1_ClearFlag(TIM1_FLAG_CC2);
}

// TIM3 interrupt routine
// display frequency counter values
void TIM3_Update_isr(void) interrupt 15
{
	char TXStr[100];
  float MEASURED_HZ, MEASURED_DUTY;
	GPIO_WriteReverse(GPIOD,GPIO_PIN_0); // int freq monitor CN4-1pin
	MEASURED_HZ = (float)fMASTER / (float)TIM1_IC_value1;
	MEASURED_DUTY = (float)TIM1_IC_value2 / (float)TIM1_IC_value1 * 100.0 ;
	sprintf(TXStr,"MEAS: %9.2f[Hz] %5.2f[pct]",MEASURED_HZ,MEASURED_DUTY);
	SerialPutString(TXStr);
	sprintf(TXStr," TIM1_IC_value1=%u  TIM1_IC_value2=%u",TIM1_IC_value1,TIM1_IC_value2);
	SerialPutString(TXStr);
	SerialPutString("\r\n");
	TIM3_ClearFlag(TIM3_FLAG_UPDATE);
}

#ifdef USE_FULL_ASSERT
void assert_failed(u8* file, u32 line)
{ 
  while (1)  	{	}
}
#endif
